@@ -21,5 +21,13 @@ except ImportError:
 
 __author__ = "Maximilian Schambach"
 
-from .core import get_mongodb, get_metric, get_uri, get_conf_databases
-from .core import get_df_query, get_df_metrics, get_df_runs, get_df_full, resolve_artifacts, get_artifact
+from .core import get_artifact
+from .core import get_conf_databases
+from .core import get_dataframe
+from .core import get_metric
+from .core import get_mongodb
+from .core import get_uri
+from .core import resolve_artifacts
+from . import environ
+
+environ.set_defaults()
