@@ -16,13 +16,18 @@
 
 import configparser
 from pathlib import Path
-from typing import Union, List, Optional
+from typing import List
+from typing import Optional
+from typing import Union
 
 import gridfs
 import pandas as pd
 from bson.objectid import ObjectId
 from pymongo import MongoClient
-from pymongo.database import Database, Collection
+from pymongo.database import Collection
+from pymongo.database import Database
+
+from mdbh import caching
 
 
 def get_uri(conf_path: Union[Path, str],
@@ -147,9 +152,9 @@ def get_metric(db: Database, id: int, metric: str) -> dict:
     return metrics.find_one({"run_id": id, "name": metric})
 
 
-def get_df_query(db: Database,
-                 query: dict,
-                 collection: Union[str, Collection]) -> pd.DataFrame:
+def _get_dataframe_query(db: Database,
+                         query: dict,
+                         collection: Union[str, Collection]) -> pd.DataFrame:
     """Get a DataFrame from a MongoDB query."""
     if type(db) != Database:
         raise TypeError("db needs to be a PyMongo Database instance. "
@@ -161,52 +166,31 @@ def get_df_query(db: Database,
     return df
 
 
-def get_df_metrics(db: Database,
-                   ids: Optional[List[int]] = None) -> pd.DataFrame:
-    """Get DataFrame with metrics from run IDs.
-
-    Args:
-        db: Database instance.
-        ids: Query run IDs. If None, all IDs are queried.
-
-    Returns:
-        DataFrame containing the metrics of queried IDs.
-    """
-    # Build query
-    query = {} if ids is None else {"run_id": {"$in": ids}}
-    return get_df_query(db, query, "metrics")
-
-
-def get_df_runs(db: Database,
-                ids: Optional[List[int]] = None) -> pd.DataFrame:
-    """Get DataFrame with run configuration from run IDs.
-
-    Args:
-        db: Database instance.
-        ids: Query run IDs. If None, all IDs are queried.
-
-    Returns:
-        DataFrame containing the run configuration of queried IDs.
-
-    """
-    query = {} if ids is None else {"_id": {"$in": ids}}
-    return get_df_query(db, query, "runs")
-
-
-def get_df_full(db: Database, return_artifacts: bool=False) -> pd.DataFrame:
+def get_dataframe(db: Database,
+                  ids: Optional[List[int]] = None,
+                  include_artifacts: bool = False) -> pd.DataFrame:
     """Get all experiments from a Sacred Database in a DataFrame.
+    Metrics and configurations are concatenated into a single DataFrame for
+    convenience.
 
     Args:
         db: DataBase instance.
 
+        ids: Optional list of run IDs to get. If None, all IDs are queried.
+
+        include_artifacts: Whether to include a column for artifacts.
+                           Artifacts can then be resolved via
+                           :func:resolve_artifacts().
+
     Returns:
         DataFrame of the corresponding Database.
-        Metrics and configurations are concatinated into a single
-        DataFrame for convenience.
     """
-    # Get all metric logs
-    df_metric = get_df_query(db, {}, "metrics")
-    df_runs = get_df_query(db, {}, "runs")
+    # Get all metric and run logs
+    query = {} if ids is None else {"run_id": {"$in": ids}}
+    df_metric = _get_dataframe_query(db, query, "metrics")
+
+    query = {} if ids is None else {"_id": {"$in": ids}}
+    df_runs = _get_dataframe_query(db, query, "runs")
 
     # Set run_id (metrics), _id (runs) as index of DataFrames for later join
     df_metric.index = df_metric['run_id'].tolist()
@@ -255,7 +239,7 @@ def get_df_full(db: Database, return_artifacts: bool=False) -> pd.DataFrame:
                      'result', 'stop_time', 'fail_trace', 'captured_out',
                      'omniboard']
 
-    if not return_artifacts:
+    if not include_artifacts:
         df_r_del_cols += ['artifacts']
 
     for i in df_m_del_cols:
@@ -271,7 +255,7 @@ def get_df_full(db: Database, return_artifacts: bool=False) -> pd.DataFrame:
     return df
 
 
-def get_artifact(db: Database, id) -> str:
+def _get_artifact_content(db: Database, id: Union[str, ObjectId]) -> bytes:
     """Retrieves file content from GridFS by ObjectID. This corresponds to the way sacred saves artifacts.
 
     Args:
@@ -285,26 +269,75 @@ def get_artifact(db: Database, id) -> str:
     gr = gridfs.GridFS(db)
     return gr.get(id).read()
 
+
 def resolve_artifacts(db: Database, df: pd.DataFrame) -> pd.DataFrame:
     """Maps the artifact column as retrieved from the database in
     get_df_full(..., return_artifacts=True) into a dictionary of the form
     {filename1: filecontent1, filename2:  filecontent2....}.
 
     Usage:
-    
-        import mdbh
+
         df = mdbh.get_df_full(db, return_artifacts=True)
         df = mdbh.resolve_artifacts(db, df)
 
     Args:
         db: DataBase instance.
-        df: The Dataframe with a artifacts column.
+        df: The Dataframe with 'artifacts' column.
 
     Returns:
         The Dataframe with a transformed column
     """
     def map_artifacts(db: Database, x):
-        return {artifact["name"]: get_artifact(db, artifact["file_id"]) for artifact in x}
+        return {artifact["name"]: _get_artifact_content(db, artifact["file_id"]) for artifact in x}
     df["artifacts"] = df["artifacts"].map(lambda x: map_artifacts(db, x))
     return df
 
+
+def get_artifact_names(db: Database, id: int) -> List[str]:
+    """Get the names of the artifacts from a run ID.
+
+    Args:
+        db: Database instance.
+        id: Run ID.
+
+    Returns:
+        List of names of the ID's artifacts.
+    """
+    df = get_dataframe(db, [id], include_artifacts=True)
+    return [a['name'] for a in df.loc[id]['artifacts']]
+
+
+def get_artifact(db: Database,
+                 id: int,
+                 name: str,
+                 force: bool = False):
+    """Get path to artifact from database and run ID.
+    The artifacts are cached by default.
+
+    Args:
+        db: DataBase instance.
+
+        id: Run ID to get artifact from.
+
+        name: Name of artifact to get.
+
+        force: Whether to force downloading of the artifact, even if cached
+                version is available.
+
+    Returns:
+
+    """
+    def get_data():
+        """Download artifact from MongoDB."""
+        df = get_dataframe(db, [id], include_artifacts=True)
+        artifacts = [a for a in df.loc[id]['artifacts'] if a['name'] == name]
+        if len(artifacts) == 0:
+            raise ValueError(f"No artifact found with name {name} for run ID {id}.")
+        if len(artifacts) > 1:
+            raise ValueError(f"Found more than one artifact with name {name} for run ID {id}.")
+        artifact = artifacts[0]
+        print(f"Downloading artifact from MongoDB.")
+        return _get_artifact_content(db, artifact['file_id'])
+
+    filename = f"{db.name}_{id}_{name}"
+    return caching.get(filename, get_data)
